#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase

from mock import patch

from pandagg.exceptions import AbsentMappingFieldError
from pandagg.node.mapping.abstract import Field
from pandagg.mapping import Keyword, Object, Text, Nested, Integer, Mapping
from tests.testing_samples.mapping_example import MAPPING, EXPECTED_MAPPING_TREE_REPR


class MappingTreeTestCase(TestCase):
    """All tree logic is tested in utils.
    Here, check that:
     - a dict mapping is correctly parsed into a tree,
     - it has the right representation.
    """

    def test_keyword_with_fields(self):
        unnamed_field = Keyword(fields={"searchable": {"type": "text"}}, fielddata=True)
        self.assertEqual(unnamed_field.body, {"fielddata": True})
        self.assertEqual(unnamed_field.fields, {"searchable": {"type": "text"}})
        named_field = unnamed_field.to_named_field(name="path.to.field")
        self.assertIsInstance(named_field, Field)
        self.assertEqual(named_field.KEY, "keyword")
        self.assertEqual(named_field.body, {"fielddata": True, "type": "keyword"})

    def test_deserialization(self):
        mapping_dict = {
            "dynamic": False,
            "properties": {
                "classification_type": {
                    "type": "keyword",
                    "fields": {"raw": {"type": "text"}},
                },
                "local_metrics": {
                    "type": "nested",
                    "dynamic": False,
                    "properties": {
                        "dataset": {
                            "dynamic": False,
                            "properties": {
                                "support_test": {"type": "integer"},
                                "support_train": {"type": "integer"},
                            },
                        }
                    },
                },
            },
        }

        m1 = Mapping(mapping_dict)

        m2 = Mapping(
            dynamic=False,
            properties={
                "classification_type": Keyword(fields={"raw": Text()}),
                "local_metrics": Nested(
                    dynamic=False,
                    properties={
                        "dataset": Object(
                            dynamic=False,
                            properties={
                                "support_test": Integer(),
                                "support_train": Integer(),
                            },
                        )
                    },
                ),
            },
        )

        m3 = Mapping(m1)

        expected_repr = """<Mapping>
_
├── classification_type                                       Keyword
│   └── raw                                                 ~ Text
└── local_metrics                                            [Nested]
    └── dataset                                              {Object}
        ├── support_test                                      Integer
        └── support_train                                     Integer
"""
        for i, m in enumerate((m1, m2, m3)):
            self.assertEqual(m.__repr__(), expected_repr, "failed at m%d" % (i + 1))
            self.assertEqual(m.to_dict(), mapping_dict, "failed at m%d" % (i + 1))

    def test_mapping_node(self):
        m = Mapping(None)
        self.assertTrue(m.is_empty())

    def test_parse_tree_from_dict(self):
        mapping_tree = Mapping(MAPPING)

        self.assertEqual(mapping_tree.__str__(), EXPECTED_MAPPING_TREE_REPR)

    def test_nesteds_applied_at_field(self):
        mapping_tree = Mapping(MAPPING)

        self.assertEqual(mapping_tree.nested_at_field("classification_type"), None)
        self.assertEqual(mapping_tree.list_nesteds_at_field("classification_type"), [])
        self.assertEqual(mapping_tree.nested_at_field("date"), None)
        self.assertEqual(mapping_tree.list_nesteds_at_field("date"), [])
        self.assertEqual(mapping_tree.nested_at_field("global_metrics"), None)
        self.assertEqual(mapping_tree.list_nesteds_at_field("global_metrics"), [])

        self.assertEqual(mapping_tree.nested_at_field("local_metrics"), "local_metrics")
        self.assertEqual(
            mapping_tree.list_nesteds_at_field("local_metrics"), ["local_metrics"]
        )
        self.assertEqual(
            mapping_tree.nested_at_field("local_metrics.dataset.support_test"),
            "local_metrics",
        )
        self.assertEqual(
            mapping_tree.list_nesteds_at_field("local_metrics.dataset.support_test"),
            ["local_metrics"],
        )

    @patch("uuid.uuid4")
    def test_resolve_path_to_id(self, uuid_mock):
        uuid_mock.side_effect = range(100)
        mapping_tree = Mapping(MAPPING)
        # do not resolve
        self.assertEqual(
            mapping_tree.resolve_path_to_id("global_metrics.non_existing_field"),
            "global_metrics.non_existing_field",
        )
        # resolve
        resolved = mapping_tree.resolve_path_to_id("classification_type")
        self.assertIn("classification_type", resolved)
        self.assertIn(resolved, mapping_tree)

        resolved = mapping_tree.resolve_path_to_id("local_metrics.dataset.support_test")
        self.assertIn("support_test", resolved)
        self.assertIn(resolved, mapping_tree)

    def test_mapping_type_of_field(self):
        mapping_tree = Mapping(MAPPING)
        with self.assertRaises(AbsentMappingFieldError):
            self.assertEqual(mapping_tree.mapping_type_of_field("yolo"), False)

        self.assertEqual(mapping_tree.mapping_type_of_field("global_metrics"), "object")
        self.assertEqual(mapping_tree.mapping_type_of_field("local_metrics"), "nested")
        self.assertEqual(
            mapping_tree.mapping_type_of_field("global_metrics.field.name.raw"),
            "keyword",
        )
        self.assertEqual(
            mapping_tree.mapping_type_of_field("local_metrics.dataset.support_test"),
            "integer",
        )

    def test_node_path(self):
        mapping_tree = Mapping(MAPPING)
        # get node by path syntax
        node = mapping_tree.get("local_metrics.dataset.support_test")
        self.assertIsInstance(node, Field)
        self.assertEqual(node.name, "support_test")
        self.assertEqual(
            mapping_tree.node_path(node.identifier),
            "local_metrics.dataset.support_test",
        )



from unittest import TestCase

from pandagg.aggs import BucketSelector


class PipelineAggNodesTestCase(TestCase):
    def test_bucket_selector(self):
        agg_node = BucketSelector(
            name="agg_name", buckets_path={"stuff": "other_agg"}, script="stuff > 100"
        )

        # test query dict
        self.assertEqual(
            agg_node.to_dict(with_name=True),
            {
                "agg_name": {
                    "bucket_selector": {
                        "buckets_path": {"stuff": "other_agg"},
                        "script": "stuff > 100",
                    }
                }
            },
        )



from .abstract import Leaf


class GeoBoundingBox(Leaf):
    KEY = "geo_bounding_box"


class GeoDistance(Leaf):
    KEY = "geo_distance"


class GeoPolygone(Leaf):
    KEY = "geo_polygon"


class GeoShape(Leaf):
    KEY = "geo_shape"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pandagg.node.aggs.abstract import AggNode

from pandagg.node.types import MAPPING_TYPES

from operator import itemgetter


def list_available_aggs_on_field(field_type):
    """For a given field type, return all aggregations that can be operated on this field.
    If WHITELISTED_MAPPING_TYPES is defined, field type must be in it. Else if BLACKLISTED_MAPPING_TYPES is defined,
    field type must not be in it.
    """
    return [
        agg_class
        for agg_class in AggNode._classes.values()
        if agg_class.valid_on_field_type(field_type)
    ]


def field_klass_init(self, mapping_tree, client, field, index):
    self._mapping_tree = mapping_tree
    self._client = client
    self._field = field
    self._index = index


def aggregator_factory(agg_klass):
    def aggregator(self, index=None, raw_output=False, query=None, **kwargs):
        node = agg_klass(name="%s_agg" % agg_klass.KEY, field=self._field, **kwargs)
        return self._operate(node, index, raw_output, query)

    aggregator.__doc__ = agg_klass.__init__.__doc__ or agg_klass.__doc__
    return aggregator


def _operate(self, agg_node, index, raw_output, query):
    index = index or self._index
    aggregation = {agg_node.name: agg_node.to_dict()}
    nesteds = self._mapping_tree.list_nesteds_at_field(self._field) or []
    for nested in nesteds:
        aggregation = {nested: {"nested": {"path": nested}, "aggs": aggregation}}

    body = {"aggs": aggregation, "size": 0}
    if query is not None:
        body["query"] = query
    raw_response = self._client.search(index=index, body=body)["aggregations"]
    for nested in nesteds:
        raw_response = raw_response[nested]
    result = list(agg_node.extract_buckets(raw_response[agg_node.name]))

    if raw_output:
        return result
    try:
        import pandas as pd
    except ImportError:
        return result
    keys = map(itemgetter(0), result)
    raw_values = map(itemgetter(1), result)
    return pd.DataFrame(index=keys, data=raw_values)


def field_type_klass_factory(field_type):
    d = {"__init__": field_klass_init, "_operate": _operate}
    for agg_klass in list_available_aggs_on_field(field_type):
        d[agg_klass.KEY] = aggregator_factory(agg_klass)
    klass = type("%sAggs" % field_type.capitalize(), (), d)
    return klass


field_classes_per_name = {
    field_type: field_type_klass_factory(field_type) for field_type in MAPPING_TYPES
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    # python 2
    from StringIO import StringIO
except ImportError:
    # python 3
    from io import StringIO

import sys

from mock import Mock
from unittest import TestCase

from pandagg.mapping import Keyword, Text, Nested, Object, Integer
from pandagg.tree.mapping import Mapping
from pandagg.interactive._field_agg_factory import field_classes_per_name
from pandagg.interactive.mapping import IMapping

from tests.testing_samples.mapping_example import MAPPING


class IMappingTestCase(TestCase):
    def test_mapping_aggregations(self):
        mapping_tree = Mapping(MAPPING)
        # check that leaves are expanded, based on 'field_name' attribute of nodes
        mapping = IMapping(mapping_tree, depth=1)
        for field_name in (
            "classification_type",
            "date",
            "global_metrics",
            "id",
            "language",
            "local_metrics",
            "workflow",
        ):
            self.assertTrue(hasattr(mapping, field_name))

        dataset = mapping.global_metrics.dataset
        self.assertEqual(
            dataset.__repr__(),
            """<Mapping subpart: global_metrics.dataset>
dataset                                                      {Object}
├── nb_classes                                                Integer
└── support_train                                             Integer
""",
        )
        # capture print statement
        captured_output = StringIO()
        sys.stdout = captured_output
        # what triggers print
        dataset()
        # restore stout
        sys.stdout = sys.__stdout__
        self.assertEqual(
            captured_output.getvalue(),
            """{
  "dynamic": false,
  "properties": {
    "nb_classes": {
      "type": "integer"
    },
    "support_train": {
      "type": "integer"
    }
  }
}
""",
        )

    def test_imapping_init(self):

        mapping_dict = {
            "dynamic": False,
            "properties": {
                "classification_type": {
                    "type": "keyword",
                    "fields": {"raw": {"type": "text"}},
                },
                "local_metrics": {
                    "type": "nested",
                    "dynamic": False,
                    "properties": {
                        "dataset": {
                            "dynamic": False,
                            "properties": {
                                "support_test": {"type": "integer"},
                                "support_train": {"type": "integer"},
                            },
                        }
                    },
                },
            },
        }

        mapping_tree = Mapping(mapping_dict)
        client_mock = Mock(spec=["search"])
        index_name = "classification_report_index_name"

        # from dict
        im1 = IMapping(mapping_dict, client=client_mock, index=index_name)
        # from tree
        im2 = IMapping(mapping_tree, client=client_mock, index=index_name)

        # from nodes
        im3 = IMapping(
            properties={
                "classification_type": Keyword(fields={"raw": Text()}),
                "local_metrics": Nested(
                    dynamic=False,
                    properties={
                        "dataset": Object(
                            dynamic=False,
                            properties={
                                "support_test": Integer(),
                                "support_train": Integer(),
                            },
                        )
                    },
                ),
            },
            dynamic=False,
            client=client_mock,
            index=index_name,
        )
        for i, m in enumerate((im1, im2, im3)):
            self.assertEqual(m._tree.to_dict(), mapping_dict, "failed at m%d" % (i + 1))
            self.assertEqual(m._index, index_name)
            self.assertIs(m._client, client_mock)

    def test_quick_agg(self):
        """Check that when reaching leaves (fields without children) leaves have the "a" attribute that can generate
        aggregations on that field type.
        """
        client_mock = Mock(spec=["search"])
        es_response_mock = {
            "_shards": {"failed": 0, "successful": 135, "total": 135},
            "aggregations": {
                "terms_agg": {
                    "buckets": [
                        {"doc_count": 25, "key": 1},
                        {"doc_count": 50, "key": 2},
                    ],
                    "doc_count_error_upper_bound": 0,
                    "sum_other_doc_count": 4,
                }
            },
            "hits": {"hits": [], "max_score": 0.0, "total": 300},
            "timed_out": False,
            "took": 30,
        }
        client_mock.search = Mock(return_value=es_response_mock)

        mapping_tree = Mapping(MAPPING)
        client_bound_mapping = IMapping(
            mapping_tree, client=client_mock, index="classification_report_index_name",
        )

        workflow_field = client_bound_mapping.workflow
        self.assertTrue(hasattr(workflow_field, "a"))
        # workflow type is keyword
        self.assertIsInstance(workflow_field.a, field_classes_per_name["keyword"])

        response = workflow_field.a.terms(
            size=20,
            raw_output=True,
            query={"term": {"classification_type": "multiclass"}},
        )
        self.assertEqual(
            response,
            [(1, {"doc_count": 25, "key": 1}), (2, {"doc_count": 50, "key": 2}),],
        )
        client_mock.search.assert_called_once()
        client_mock.search.assert_called_with(
            body={
                "aggs": {"terms_agg": {"terms": {"field": "workflow", "size": 20}}},
                "size": 0,
                "query": {"term": {"classification_type": "multiclass"}},
            },
            index="classification_report_index_name",
        )

    def test_quick_agg_nested(self):
        """Check that when reaching leaves (fields without children) leaves have the "a" attribute that can generate
        aggregations on that field type, applying nested if necessary.
        """
        client_mock = Mock(spec=["search"])
        es_response_mock = {
            "_shards": {"failed": 0, "successful": 135, "total": 135},
            "aggregations": {"local_metrics": {"avg_agg": {"value": 23},},},
            "hits": {"hits": [], "max_score": 0.0, "total": 300},
            "timed_out": False,
            "took": 30,
        }
        client_mock.search = Mock(return_value=es_response_mock)

        mapping_tree = Mapping(MAPPING)
        client_bound_mapping = IMapping(
            mapping_tree, client=client_mock, index="classification_report_index_name",
        )

        local_train_support = client_bound_mapping.local_metrics.dataset.support_train
        self.assertTrue(hasattr(local_train_support, "a"))
        self.assertIsInstance(local_train_support.a, field_classes_per_name["integer"])

        response = local_train_support.a.avg(
            size=20,
            raw_output=True,
            query={"term": {"classification_type": "multiclass"}},
        )
        self.assertEqual(
            response, [(None, {"value": 23}),],
        )
        client_mock.search.assert_called_once()
        client_mock.search.assert_called_with(
            body={
                "aggs": {
                    "local_metrics": {
                        "nested": {"path": "local_metrics"},
                        "aggs": {
                            "avg_agg": {
                                "avg": {
                                    "field": "local_metrics.dataset.support_train",
                                    "size": 20,
                                }
                            }
                        },
                    }
                },
                "size": 0,
                "query": {"term": {"classification_type": "multiclass"}},
            },
            index="classification_report_index_name",
        )



from .abstract import Compound


class Nested(Compound):
    KEY = "nested"


class HasChild(Compound):
    KEY = "has_child"


class HasParent(Compound):
    KEY = "has_parent"


class ParentId(Compound):
    KEY = "parent_id"



from .abstract import Leaf


class Intervals(Leaf):
    KEY = "intervals"


class Match(Leaf):
    KEY = "match"


class MatchBoolPrefix(Leaf):
    KEY = "match_bool_prefix"


class MatchPhrase(Leaf):
    KEY = "match_phrase"


class MatchPhrasePrefix(Leaf):
    KEY = "match_phrase_prefix"


class MultiMatch(Leaf):
    KEY = "multi_match"


class Common(Leaf):
    KEY = "common"


class QueryString(Leaf):
    KEY = "query_string"


class SimpleQueryString(Leaf):
    KEY = "simple_string"



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals
from builtins import str as text

import uuid
from future.utils import iteritems

from lighttree import Node as OriginalNode
from six import add_metaclass

from pandagg.utils import DslMeta, get_dsl_class


@add_metaclass(DslMeta)
class Node(OriginalNode):

    KEY = None
    _type_name = None

    NID_SIZE = 8

    def __init__(self, identifier=None):
        if identifier is None:
            identifier = self._craft_identifier()
        super(Node, self).__init__(identifier=identifier)

    @property
    def _identifier_prefix(self):
        return ""

    def _craft_identifier(self):
        return "%s%s" % (self._identifier_prefix, text(uuid.uuid4())[: self.NID_SIZE])

    get_dsl_class = classmethod(get_dsl_class)

    @staticmethod
    def expand__to_dot(params):
        nparams = {}
        for pname, pvalue in iteritems(params):
            if "__" in pname:
                pname = pname.replace("__", ".")
            nparams[pname] = pvalue
        return nparams



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# =============================================================================
#                                   IMPORTS
# =============================================================================

from unittest import TestCase
from lighttree.exceptions import MultipleRootError, NotFoundNodeError
from mock import patch

from pandagg.tree.aggs.aggs import Aggs
from pandagg.exceptions import (
    AbsentMappingFieldError,
    InvalidOperationMappingFieldError,
)
from pandagg.aggs import DateHistogram, Terms, Filter, Avg, Min

import tests.testing_samples.data_sample as sample

from tests.testing_samples.mapping_example import MAPPING


def to_id_set(nodes):
    return {n.identifier for n in nodes}


class AggTestCase(TestCase):
    def setUp(self):
        patcher = patch("uuid.uuid4", side_effect=range(1000))
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_deserialize_nodes_with_subaggs(self):
        expected = {
            "genres": {
                "terms": {"field": "genres", "size": 3},
                "aggs": {
                    "movie_decade": {
                        "date_histogram": {"field": "year", "fixed_interval": "3650d"}
                    }
                },
            }
        }
        agg1 = Aggs(expected)
        agg2 = Aggs(
            Terms(
                "genres",
                field="genres",
                size=3,
                aggs=DateHistogram(
                    name="movie_decade", field="year", fixed_interval="3650d"
                ),
            )
        )
        agg3 = Aggs(
            Terms(
                "genres",
                field="genres",
                size=3,
                aggs=[
                    DateHistogram(
                        name="movie_decade", field="year", fixed_interval="3650d"
                    )
                ],
            )
        )
        agg4 = Aggs(
            Terms(
                "genres",
                field="genres",
                size=3,
                aggs={
                    "movie_decade": {
                        "date_histogram": {"field": "year", "fixed_interval": "3650d"}
                    }
                },
            )
        )
        agg5 = Aggs(
            {
                "genres": {
                    "terms": {"field": "genres", "size": 3},
                    "aggs": DateHistogram(
                        name="movie_decade", field="year", fixed_interval="3650d"
                    ),
                }
            }
        )
        for a in (agg1, agg2, agg3, agg4, agg5):
            self.assertEqual(a.to_dict(), expected)

    def test_add_node_with_mapping(self):
        with_mapping = Aggs(mapping=MAPPING, nested_autocorrect=True)
        self.assertEqual(len(with_mapping.list()), 0)

        # add regular node
        with_mapping = with_mapping.aggs(Terms("workflow", field="workflow"))
        self.assertEqual(
            with_mapping.to_dict(), {"workflow": {"terms": {"field": "workflow"}}}
        )

        # try to add field aggregation on non-existing field will fail
        with self.assertRaises(AbsentMappingFieldError):
            with_mapping.aggs(
                Terms("imaginary_agg", field="imaginary_field"),
                insert_below="workflow",
            )
        self.assertEqual(len(with_mapping.list()), 1)

        # try to add aggregation on a non-compatible field will fail
        with self.assertRaises(InvalidOperationMappingFieldError):
            with_mapping.aggs(
                Avg("average_of_string", field="classification_type"),
                insert_below="workflow",
            )
        self.assertEqual(len(with_mapping.list()), 1)

        # add field aggregation on field passing through nested will automatically add nested
        with_mapping = with_mapping.aggs(
            Avg("local_f1_score", field="local_metrics.performance.test.f1_score"),
            insert_below="workflow",
        )
        self.assertEqual(
            with_mapping.to_dict(),
            {
                "workflow": {
                    "aggs": {
                        "nested_below_workflow": {
                            "aggs": {
                                "local_f1_score": {
                                    "avg": {
                                        "field": "local_metrics.performance.test.f1_score"
                                    }
                                }
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "terms": {"field": "workflow"},
                }
            },
        )
        self.assertIn("nested_below_workflow", with_mapping)
        nested_node = with_mapping.get("nested_below_workflow")
        self.assertEqual(nested_node.KEY, "nested")
        self.assertEqual(nested_node.path, "local_metrics")

        # add other agg requiring nested will reuse nested agg as parent
        with_mapping = with_mapping.aggs(
            Avg("local_precision", field="local_metrics.performance.test.precision"),
            insert_below="workflow",
        )
        self.assertEqual(
            with_mapping.to_dict(),
            {
                "workflow": {
                    "aggs": {
                        "nested_below_workflow": {
                            "aggs": {
                                "local_f1_score": {
                                    "avg": {
                                        "field": "local_metrics.performance.test.f1_score"
                                    }
                                },
                                "local_precision": {
                                    "avg": {
                                        "field": "local_metrics.performance.test.precision"
                                    }
                                },
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "terms": {"field": "workflow"},
                }
            },
        )
        self.assertEqual(len(with_mapping.list()), 4)

        # add under a nested parent a field aggregation that requires to be located under root will automatically
        # add reverse-nested
        with_mapping = with_mapping.aggs(
            Terms("language_terms", field="language"),
            insert_below="nested_below_workflow",
        )
        self.assertEqual(len(with_mapping.list()), 6)
        self.assertEqual(
            with_mapping.to_dict(),
            {
                "workflow": {
                    "aggs": {
                        "nested_below_workflow": {
                            "aggs": {
                                "local_f1_score": {
                                    "avg": {
                                        "field": "local_metrics.performance.test.f1_score"
                                    }
                                },
                                "local_precision": {
                                    "avg": {
                                        "field": "local_metrics.performance.test.precision"
                                    }
                                },
                                "reverse_nested_below_nested_below_workflow": {
                                    "aggs": {
                                        "language_terms": {
                                            "terms": {"field": "language"}
                                        }
                                    },
                                    "reverse_nested": {},
                                },
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "terms": {"field": "workflow"},
                }
            },
        )

    # TODO - finish these tests (reverse nested)
    def test_paste_tree_with_mapping(self):
        # with explicit nested
        initial_agg_1 = Aggs(
            {
                "week": {
                    "date_histogram": {
                        "field": "date",
                        "format": "yyyy-MM-dd",
                        "interval": "1w",
                    }
                }
            },
            mapping=MAPPING,
        )
        self.assertEqual(to_id_set(initial_agg_1.list()), {"week"})
        pasted_agg_1 = Aggs(
            {
                "nested_below_week": {
                    "nested": {"path": "local_metrics"},
                    "aggs": {
                        "local_metrics.field_class.name": {
                            "terms": {
                                "field": "local_metrics.field_class.name",
                                "size": 10,
                            }
                        }
                    },
                }
            }
        )
        self.assertEqual(
            to_id_set(pasted_agg_1.list()),
            {"nested_below_week", "local_metrics.field_class.name"},
        )

        initial_agg_1.insert_tree(pasted_agg_1, "week")
        self.assertEqual(
            to_id_set(initial_agg_1.list()),
            {"week", "nested_below_week", "local_metrics.field_class.name"},
        )
        self.assertEqual(
            initial_agg_1.to_dict(),
            {
                "week": {
                    "date_histogram": {
                        "field": "date",
                        "format": "yyyy-MM-dd",
                        "interval": "1w",
                    },
                    "aggs": {
                        "nested_below_week": {
                            "nested": {"path": "local_metrics"},
                            "aggs": {
                                "local_metrics.field_class.name": {
                                    "terms": {
                                        "field": "local_metrics.field_class.name",
                                        "size": 10,
                                    }
                                }
                            },
                        }
                    },
                }
            },
        )

        # without explicit nested
        initial_agg_2 = Aggs(
            {
                "week": {
                    "date_histogram": {
                        "field": "date",
                        "format": "yyyy-MM-dd",
                        "interval": "1w",
                    }
                }
            },
            mapping=MAPPING,
            nested_autocorrect=True,
        )
        self.assertEqual(to_id_set(initial_agg_2.list()), {"week"})
        pasted_agg_2 = Aggs(
            {
                "local_metrics.field_class.name": {
                    "terms": {"field": "local_metrics.field_class.name", "size": 10}
                }
            }
        )
        self.assertEqual(
            to_id_set(pasted_agg_2.list()), {"local_metrics.field_class.name"}
        )

        initial_agg_2.insert_tree(pasted_agg_2, "week")
        self.assertEqual(
            to_id_set(initial_agg_2.list()),
            {"week", "nested_below_week", "local_metrics.field_class.name"},
        )
        self.assertEqual(
            initial_agg_2.to_dict(),
            {
                "week": {
                    "date_histogram": {
                        "field": "date",
                        "format": "yyyy-MM-dd",
                        "interval": "1w",
                    },
                    "aggs": {
                        "nested_below_week": {
                            "nested": {"path": "local_metrics"},
                            "aggs": {
                                "local_metrics.field_class.name": {
                                    "terms": {
                                        "field": "local_metrics.field_class.name",
                                        "size": 10,
                                    }
                                }
                            },
                        }
                    },
                }
            },
        )

    def test_insert_tree_without_mapping(self):
        # with explicit nested
        initial_agg_1 = Aggs(
            {
                "week": {
                    "date_histogram": {
                        "field": "date",
                        "format": "yyyy-MM-dd",
                        "interval": "1w",
                    }
                }
            },
        )
        self.assertEqual({n.identifier for n in initial_agg_1.list()}, {"week"})

        pasted_agg_1 = Aggs(
            {
                "nested_below_week": {
                    "nested": {"path": "local_metrics"},
                    "aggs": {
                        "local_metrics.field_class.name": {
                            "terms": {
                                "field": "local_metrics.field_class.name",
                                "size": 10,
                            }
                        }
                    },
                }
            }
        )
        self.assertEqual(
            to_id_set(pasted_agg_1.list()),
            {"nested_below_week", "local_metrics.field_class.name"},
        )

        initial_agg_1.insert_tree(pasted_agg_1, "week")
        self.assertEqual(
            to_id_set(initial_agg_1.list()),
            {"week", "nested_below_week", "local_metrics.field_class.name"},
        )
        self.assertEqual(
            initial_agg_1.to_dict(),
            {
                "week": {
                    "date_histogram": {
                        "field": "date",
                        "format": "yyyy-MM-dd",
                        "interval": "1w",
                    },
                    "aggs": {
                        "nested_below_week": {
                            "nested": {"path": "local_metrics"},
                            "aggs": {
                                "local_metrics.field_class.name": {
                                    "terms": {
                                        "field": "local_metrics.field_class.name",
                                        "size": 10,
                                    }
                                }
                            },
                        }
                    },
                }
            },
        )

    def test_interpret_agg_string(self):
        some_agg = Aggs()
        some_agg = some_agg.aggs("some_field", insert_below=None)
        self.assertEqual(
            some_agg.to_dict(), {"some_field": {"terms": {"field": "some_field"}}}
        )

        # with default size
        some_agg = Aggs()
        some_agg = some_agg.aggs("some_field", insert_below=None, size=10)
        self.assertEqual(
            some_agg.to_dict(),
            {"some_field": {"terms": {"field": "some_field", "size": 10}}},
        )

        # with parent
        some_agg = Aggs(
            {"root_agg_name": {"terms": {"field": "some_field", "size": 5}}}
        )
        some_agg = some_agg.aggs("child_field", insert_below="root_agg_name")
        self.assertEqual(
            some_agg.to_dict(),
            {
                "root_agg_name": {
                    "aggs": {"child_field": {"terms": {"field": "child_field"}}},
                    "terms": {"field": "some_field", "size": 5},
                }
            },
        )

        # with required nested
        some_agg = Aggs(
            {"term_workflow": {"terms": {"field": "workflow", "size": 5}}},
            mapping=MAPPING,
            nested_autocorrect=True,
        )
        some_agg = some_agg.aggs(
            "local_metrics.field_class.name", insert_below="term_workflow"
        )
        self.assertEqual(
            some_agg.to_dict(),
            {
                "term_workflow": {
                    "aggs": {
                        "nested_below_term_workflow": {
                            "aggs": {
                                "local_metrics.field_class.name": {
                                    "terms": {"field": "local_metrics.field_class.name"}
                                }
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "terms": {"field": "workflow", "size": 5},
                }
            },
        )

    def test_aggs(self):
        node = Terms(name="some_name", field="some_field", size=10)
        some_agg = Aggs().aggs(node, insert_below=None)
        self.assertEqual(
            some_agg.to_dict(),
            {"some_name": {"terms": {"field": "some_field", "size": 10}}},
        )
        # with parent with required nested
        some_agg = Aggs(
            {"term_workflow": {"terms": {"field": "workflow", "size": 5}}},
            mapping=MAPPING,
            nested_autocorrect=True,
        )
        node = Avg(name="min_local_f1", field="local_metrics.performance.test.f1_score")
        some_agg = some_agg.aggs(node, insert_below="term_workflow")
        self.assertEqual(
            some_agg.to_dict(),
            {
                "term_workflow": {
                    "aggs": {
                        "nested_below_term_workflow": {
                            "aggs": {
                                "min_local_f1": {
                                    "avg": {
                                        "field": "local_metrics.performance.test.f1_score"
                                    }
                                }
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "terms": {"field": "workflow", "size": 5},
                }
            },
        )

    def test_aggs_at_root(self):
        a = (
            Aggs()
            .aggs("one", "terms", field="terms_one")
            .aggs("two", "terms", field="terms_two", at_root=True)
        )
        self.assertEqual(
            a.to_dict(),
            {
                "one": {"terms": {"field": "terms_one"}},
                "two": {"terms": {"field": "terms_two"}},
            },
        )

        # not at root: default behavior
        a = (
            Aggs()
            .aggs("one", "terms", field="terms_one")
            .aggs("two", "terms", field="terms_two")
        )
        self.assertEqual(
            a.to_dict(),
            {
                "one": {
                    "terms": {"field": "terms_one"},
                    "aggs": {"two": {"terms": {"field": "terms_two"}}},
                },
            },
        )

    def test_validate_aggs_parent_id(self):
        """
        <Aggregation>
        classification_type
        └── global_metrics.field.name
            ├── avg_f1_micro
            └── avg_nb_classes
        """
        my_agg = Aggs(sample.EXPECTED_AGG_QUERY, mapping=MAPPING)

        with self.assertRaises(ValueError) as e:
            my_agg._validate_aggs_parent_id(pid=None)
        self.assertEqual(
            e.exception.args,
            (
                "Declaration is ambiguous, you must declare the node id under which these "
                "aggregations should be placed.",
            ),
        )

        with self.assertRaises(ValueError) as e:
            my_agg._validate_aggs_parent_id("avg_f1_micro")
        self.assertEqual(
            e.exception.args, ("Node id <avg_f1_micro> is not a bucket aggregation.",)
        )

        self.assertEqual(
            my_agg._validate_aggs_parent_id("global_metrics.field.name"),
            "global_metrics.field.name",
        )

        with self.assertRaises(NotFoundNodeError) as e:
            my_agg._validate_aggs_parent_id("non-existing-node")
        self.assertEqual(
            e.exception.args, ("Node id <non-existing-node> doesn't exist in tree",)
        )

        # linear agg
        my_agg.drop_node("avg_f1_micro")
        my_agg.drop_node("avg_nb_classes")
        """
        <Aggregation>
        classification_type
        └── global_metrics.field.name
        """
        self.assertEqual(
            my_agg._validate_aggs_parent_id(None), "global_metrics.field.name"
        )

        # empty agg
        agg = Aggs()
        self.assertEqual(agg._validate_aggs_parent_id(None), None)

        # TODO - pipeline aggregation under metric agg

    def test_init_from_node_hierarchy(self):
        node_hierarchy = sample.get_node_hierarchy()

        agg = Aggs(node_hierarchy, mapping=MAPPING)
        self.assertEqual(agg.to_dict(), sample.EXPECTED_AGG_QUERY)

        # with nested
        node_hierarchy = DateHistogram(
            name="week",
            field="date",
            interval="1w",
            aggs=[
                Terms(
                    name="local_metrics.field_class.name",
                    field="local_metrics.field_class.name",
                    size=10,
                    aggs=[
                        Min(
                            name="min_f1_score",
                            field="local_metrics.performance.test.f1_score",
                        )
                    ],
                )
            ],
        )
        agg = Aggs(node_hierarchy, mapping=MAPPING, nested_autocorrect=True)
        self.assertEqual(
            agg.to_dict(),
            {
                "week": {
                    "aggs": {
                        "nested_below_week": {
                            "aggs": {
                                "local_metrics.field_class.name": {
                                    "aggs": {
                                        "min_f1_score": {
                                            "min": {
                                                "field": "local_metrics.performance.test.f1_score"
                                            }
                                        }
                                    },
                                    "terms": {
                                        "field": "local_metrics.field_class.name",
                                        "size": 10,
                                    },
                                }
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "date_histogram": {"field": "date", "interval": "1w"},
                }
            },
        )
        self.assertEqual(
            agg.to_dict(),
            {
                "week": {
                    "aggs": {
                        "nested_below_week": {
                            "aggs": {
                                "local_metrics.field_class.name": {
                                    "aggs": {
                                        "min_f1_score": {
                                            "min": {
                                                "field": "local_metrics.performance.test.f1_score"
                                            }
                                        }
                                    },
                                    "terms": {
                                        "field": "local_metrics.field_class.name",
                                        "size": 10,
                                    },
                                }
                            },
                            "nested": {"path": "local_metrics"},
                        }
                    },
                    "date_histogram": {"field": "date", "interval": "1w"},
                }
            },
        )

    def test_agg_init(self):
        agg = sample.get_wrapper_declared_agg()
        self.assertEqual(agg.to_dict(), sample.EXPECTED_AGG_QUERY)

    def test_groupby_args_syntax(self):
        a = Aggs().groupby("some_name", "terms", field="some_field")
        self.assertEqual(a.to_dict(), {"some_name": {"terms": {"field": "some_field"}}})

    def test_groupby_at_root(self):
        a = (
            Aggs()
            .groupby("one", "terms", field="terms_one")
            .groupby("two", "terms", field="terms_two", at_root=True)
        )
        self.assertEqual(
            a.to_dict(),
            {
                "two": {
                    "terms": {"field": "terms_two"},
                    "aggs": {"one": {"terms": {"field": "terms_one"}}},
                },
            },
        )

        # not at root: default behavior
        a = (
            Aggs()
            .groupby("one", "terms", field="terms_one")
            .groupby("two", "terms", field="terms_two")
        )
        self.assertEqual(
            a.to_dict(),
            {
                "one": {
                    "terms": {"field": "terms_one"},
                    "aggs": {"two": {"terms": {"field": "terms_two"}}},
                },
            },
        )

    def test_groupby_insert_below(self):
        a1 = Aggs(
            Terms("A", field="A", aggs=[Terms("B", field="B"), Terms("C", field="C")])
        )
        self.assertEqual(
            a1.to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "C": {"terms": {"field": "C"}},
                        "B": {"terms": {"field": "B"}},
                    },
                }
            },
        )

        self.assertEqual(
            a1.groupby(Terms("D", field="D"), insert_below="A").to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "D": {
                            "terms": {"field": "D"},
                            "aggs": {
                                "B": {"terms": {"field": "B"}},
                                "C": {"terms": {"field": "C"}},
                            },
                        }
                    },
                }
            },
        )
        self.assertEqual(
            a1.groupby(
                [Terms("D", field="D"), Terms("E", field="E")], insert_below="A"
            ).to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "D": {
                            "terms": {"field": "D"},
                            "aggs": {
                                "E": {
                                    "terms": {"field": "E"},
                                    "aggs": {
                                        "C": {"terms": {"field": "C"}},
                                        "B": {"terms": {"field": "B"}},
                                    },
                                }
                            },
                        }
                    },
                }
            },
        )
        self.assertEqual(
            a1.groupby(
                Terms("D", field="D", aggs=Terms("E", field="E")), insert_below="A"
            ).to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "D": {
                            "terms": {"field": "D"},
                            "aggs": {
                                "E": {
                                    "terms": {"field": "E"},
                                    "aggs": {
                                        "B": {"terms": {"field": "B"}},
                                        "C": {"terms": {"field": "C"}},
                                    },
                                }
                            },
                        }
                    },
                }
            },
        )

    def test_groupby_insert_above(self):
        a1 = Aggs(
            Terms("A", field="A", aggs=[Terms("B", field="B"), Terms("C", field="C")])
        )
        self.assertEqual(
            a1.to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "B": {"terms": {"field": "B"}},
                        "C": {"terms": {"field": "C"}},
                    },
                }
            },
        )

        self.assertEqual(
            a1.groupby(Terms("D", field="D"), insert_above="B").to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "C": {"terms": {"field": "C"}},
                        "D": {
                            "terms": {"field": "D"},
                            "aggs": {"B": {"terms": {"field": "B"}}},
                        },
                    },
                }
            },
        )
        self.assertEqual(
            a1.groupby(
                [Terms("D", field="D"), Terms("E", field="E")], insert_above="B"
            ).to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "C": {"terms": {"field": "C"}},
                        "D": {
                            "terms": {"field": "D"},
                            "aggs": {
                                "E": {
                                    "terms": {"field": "E"},
                                    "aggs": {"B": {"terms": {"field": "B"}}},
                                }
                            },
                        },
                    },
                }
            },
        )
        self.assertEqual(
            a1.groupby(
                Terms("D", field="D", aggs=Terms("E", field="E")), insert_above="B"
            ).to_dict(),
            {
                "A": {
                    "aggs": {
                        "C": {"terms": {"field": "C"}},
                        "D": {
                            "aggs": {
                                "E": {
                                    "aggs": {"B": {"terms": {"field": "B"}}},
                                    "terms": {"field": "E"},
                                }
                            },
                            "terms": {"field": "D"},
                        },
                    },
                    "terms": {"field": "A"},
                }
            },
        )
        # above root
        self.assertEqual(
            a1.groupby(
                Terms("D", field="D", aggs=Terms("E", field="E")), insert_above="A"
            ).to_dict(),
            {
                "D": {
                    "terms": {"field": "D"},
                    "aggs": {
                        "E": {
                            "terms": {"field": "E"},
                            "aggs": {
                                "A": {
                                    "terms": {"field": "A"},
                                    "aggs": {
                                        "B": {"terms": {"field": "B"}},
                                        "C": {"terms": {"field": "C"}},
                                    },
                                }
                            },
                        }
                    },
                }
            },
        )

    def test_agg_insert_below(self):
        a1 = Aggs(
            Terms("A", field="A", aggs=[Terms("B", field="B"), Terms("C", field="C")])
        )
        self.assertEqual(
            a1.to_dict(),
            {
                "A": {
                    "terms": {"field": "A"},
                    "aggs": {
                        "C": {"terms": {"field": "C"}},
                        "B": {"terms": {"field": "B"}},
                    },
                }
            },
        )

        self.assertEqual(
            a1.aggs(Terms("D", field="D"), insert_below="A").to_dict(),
            {
                "A": {
                    "aggs": {
                        "B": {"terms": {"field": "B"}},
                        "C": {"terms": {"field": "C"}},
                        "D": {"terms": {"field": "D"}},
                    },
                    "terms": {"field": "A"},
                }
            },
        )
        self.assertEqual(
            a1.aggs(
                [Terms("D", field="D"), Terms("E", field="E")], insert_below="A"
            ).to_dict(),
            {
                "A": {
                    "aggs": {
                        "B": {"terms": {"field": "B"}},
                        "C": {"terms": {"field": "C"}},
                        "D": {"terms": {"field": "D"}},
                        "E": {"terms": {"field": "E"}},
                    },
                    "terms": {"field": "A"},
                }
            },
        )

    def test_applied_nested_path_at_node(self):
        """ Check that correct nested path is detected at node levels:
        week
        └── nested_below_week
            └── local_metrics.field_class.name
                ├── avg_f1_score
                ├── max_f1_score
                └── min_f1_score
        """
        node_hierarchy = DateHistogram(
            name="week",
            field="date",
            interval="1w",
            aggs=[
                Terms(
                    name="local_metrics.field_class.name",
                    field="local_metrics.field_class.name",
                    size=10,
                    aggs=[
                        Min(
                            name="min_f1_score",
                            field="local_metrics.performance.test.f1_score",
                        )
                    ],
                )
            ],
        )
        agg = Aggs(node_hierarchy, mapping=MAPPING, nested_autocorrect=True)

        self.assertEqual(agg.applied_nested_path_at_node("week"), None)
        for nid in (
            "nested_below_week",
            "local_metrics.field_class.name",
            "min_f1_score",
        ):
            self.assertEqual(agg.applied_nested_path_at_node(nid), "local_metrics")

    def test_deepest_linear_agg(self):
        # deepest_linear_bucket_agg
        """
        week
        └── nested_below_week
            └── local_metrics.field_class.name   <----- HERE because then metric aggregation
                └── avg_f1_score
        """
        node_hierarchy = DateHistogram(
            name="week",
            field="date",
            interval="1w",
            aggs=[
                Terms(
                    name="local_metrics.field_class.name",
                    field="local_metrics.field_class.name",
                    size=10,
                    aggs=[
                        Min(
                            name="min_f1_score",
                            field="local_metrics.performance.test.f1_score",
                        )
                    ],
                )
            ],
        )
        agg = Aggs(node_hierarchy, mapping=MAPPING, nested_autocorrect=True)
        self.assertEqual(
            agg.deepest_linear_bucket_agg, "local_metrics.field_class.name"
        )

        # week is last bucket linear bucket
        node_hierarchy_2 = DateHistogram(
            name="week",
            field="date",
            interval="1w",
            aggs=[
                Terms(
                    name="local_metrics.field_class.name",
                    field="local_metrics.field_class.name",
                    size=10,
                ),
                Filter(
                    name="f1_score_above_threshold",
                    filter={
                        "range": {
                            "local_metrics.performance.test.f1_score": {"gte": 0.5}
                        }
                    },
                ),
            ],
        )
        agg2 = Aggs(node_hierarchy_2, mapping=MAPPING, nested_autocorrect=True)
        self.assertEqual(agg2.deepest_linear_bucket_agg, "week")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = "0.0.8"

import os

from setuptools import setup
from setuptools import find_packages


here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, "README.md")).read()

tests_require = [
    "coverage",
    "pytest",
    "mock",
    "pandas>=0.24.2",
]

install_requires = [
    "future",
    "lighttree==0.0.6",
    "elasticsearch>=7.1.0,<8.0.0",
]

extras_require = {"test": tests_require, "pandas": ["pandas>=0.24.2"]}


setup(
    name="pandagg",
    version=__version__,
    description="Python package provided to make elasticsearch aggregations and queries easy.",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Léonard Binet",
    author_email="leonardbinet@gmail.com",
    url="https://github.com/alkemics/pandagg",
    keywords="elasticsearch aggregation pandas",
    packages=find_packages(),
    include_package_data=True,
    test_suite="pandagg.tests",
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=extras_require,
)



from copy import deepcopy

from mock import patch

from pandagg.search import Search
from pandagg.aggs import Aggs
from pandagg.query import Query, Bool, Match
from pandagg.utils import ordered
from tests import PandaggTestCase


class SearchTestCase(PandaggTestCase):
    def setUp(self):
        patcher = patch("uuid.uuid4", side_effect=range(1000))
        patcher.start()
        self.addCleanup(patcher.stop)

    def test_expand__to_dot_is_respected(self):
        s = Search().query("match", a__b=42, _expand__to_dot=False)

        self.assertEqual({"query": {"match": {"a__b": {"query": 42}}}}, s.to_dict())

    def test_search_query_combines_query(self):
        s = Search()

        s2 = s.query("match", f=42)
        self.assertEqual(s2._query.to_dict(), Query(Match(f=42)).to_dict())
        self.assertEqual(s._query.to_dict(), None)

        s3 = s2.query("match", f=43)
        self.assertEqual(s2._query.to_dict(), Query(Match(f=42)).to_dict())
        self.assertEqual(
            ordered(s3._query.to_dict()),
            ordered(Query(Bool(must=[Match(f=42), Match(f=43)])).to_dict()),
        )

    def test_using(self):
        o = object()
        o2 = object()
        s = Search(using=o)
        self.assertIs(s._using, o)
        s2 = s.using(o2)
        self.assertIs(s._using, o)
        self.assertIs(s2._using, o2)

    def test_query_always_returns_search(self):
        s = Search()
        self.assertIsInstance(s.query("match", f=42), Search)

    def test_source_copied_on_clone(self):
        s = Search().source(False)

        self.assertEqual(s._clone()._source, s._source)
        self.assertIs(s._clone()._source, False)

        s2 = Search().source([])
        assert s2._clone()._source == s2._source
        assert s2._source == []

        s3 = Search().source(["some", "fields"])
        assert s3._clone()._source == s3._source
        assert s3._clone()._source == ["some", "fields"]

    def test_copy_clones(self):
        from copy import copy

        s1 = Search().source(["some", "fields"])
        s2 = copy(s1)

        assert s1 == s2
        assert s1 is not s2

    def test_aggs_allow_two_metric(self):
        s = Search()

        s = s.aggs([Aggs("a", "max", field="a"), Aggs("b", "max", field="b")])

        self.assertEqual(
            s.to_dict(),
            {"aggs": {"a": {"max": {"field": "a"}}, "b": {"max": {"field": "b"}}}},
        )

    def test_search_index(self):
        s = Search(index="i")
        assert s._index == ["i"]
        s = s.index("i2")
        assert s._index == ["i", "i2"]
        s = s.index(u"i3")
        assert s._index == ["i", "i2", "i3"]
        s = s.index()
        assert s._index is None
        s = Search(index=("i", "i2"))
        assert s._index == ["i", "i2"]
        s = Search(index=["i", "i2"])
        assert s._index == ["i", "i2"]
        s = Search()
        s = s.index("i", "i2")
        assert s._index == ["i", "i2"]
        s2 = s.index("i3")
        assert s._index == ["i", "i2"]
        assert s2._index == ["i", "i2", "i3"]
        s = Search()
        s = s.index(["i", "i2"], "i3")
        assert s._index == ["i", "i2", "i3"]
        s2 = s.index("i4")
        assert s._index == ["i", "i2", "i3"]
        assert s2._index == ["i", "i2", "i3", "i4"]
        s2 = s.index(["i4"])
        assert s2._index == ["i", "i2", "i3", "i4"]
        s2 = s.index(("i4", "i5"))
        assert s2._index == ["i", "i2", "i3", "i4", "i5"]

    def test_sort(self):
        s = Search()
        s = s.sort("fielda", "-fieldb")

        assert ["fielda", {"fieldb": {"order": "desc"}}] == s._sort
        assert {"sort": ["fielda", {"fieldb": {"order": "desc"}}]} == s.to_dict()

        s = s.sort()
        assert [] == s._sort
        assert Search().to_dict() == s.to_dict()

    def test_slice(self):
        s = Search()
        assert {"from": 3, "size": 7} == s[3:10].to_dict()
        assert {"from": 0, "size": 5} == s[:5].to_dict()
        assert {"from": 3, "size": 10} == s[3:].to_dict()
        assert {"from": 0, "size": 0} == s[0:0].to_dict()

    def test_index(self):
        s = Search()
        assert {"from": 3, "size": 1} == s[3].to_dict()

    def test_search_to_dict(self):
        s = Search()
        assert {} == s.to_dict()

        s = s.query("match", f=42)
        assert {"query": {"match": {"f": {"query": 42}}}} == s.to_dict()

        assert {"query": {"match": {"f": {"query": 42}}}, "size": 10} == s.to_dict(
            size=10
        )

        s = s.aggs("per_tag", "terms", field="f").aggs(
            "max_score", "max", field="score"
        )
        d = {
            "aggs": {
                "per_tag": {
                    "terms": {"field": "f"},
                    "aggs": {"max_score": {"max": {"field": "score"}}},
                }
            },
            "query": {"match": {"f": {"query": 42}}},
        }
        self.assertEqual(d, s.to_dict())

        s = Search().params(size=5)
        assert {"size": 5} == s.to_dict()
        s = s.params(from_=42)
        assert {"size": 5, "from": 42} == s.to_dict()

    def test_complex_example(self):
        s = (
            Search()
            .query("match", title="python")
            .must_not("match", title="ruby")
            .should(
                Query("term", category="meetup"), Query("term", category="conference")
            )
            .post_filter("terms", tags=["prague", "czech"])
            .script_fields(more_attendees="doc['attendees'].value + 42")
            .aggs("per_country", "terms", field="country")
            .aggs("avg_attendees", "avg", field="attendees")
            .bool(minimum_should_match=2)
            .highlight_options(order="score")
            .highlight("title", "body", fragment_size=50)
        )

        self.assertEqual(
            ordered(
                {
                    "aggs": {
                        "per_country": {
                            "aggs": {"avg_attendees": {"avg": {"field": "attendees"}}},
                            "terms": {"field": "country"},
                        }
                    },
                    "highlight": {
                        "fields": {
                            "body": {"fragment_size": 50},
                            "title": {"fragment_size": 50},
                        },
                        "order": "score",
                    },
                    "post_filter": {"terms": {"tags": ["prague", "czech"]}},
                    "query": {
                        "bool": {
                            "minimum_should_match": 2,
                            "must": [{"match": {"title": {"query": "python"}}}],
                            "must_not": [{"match": {"title": {"query": "ruby"}}}],
                            "should": [
                                {"term": {"category": {"value": "conference"}}},
                                {"term": {"category": {"value": "meetup"}}},
                            ],
                        }
                    },
                    "script_fields": {
                        "more_attendees": {"script": "doc['attendees'].value + 42"}
                    },
                }
            ),
            ordered(s.to_dict()),
        )

    def test_reverse(self):
        d = {
            "aggs": {
                "per_country": {
                    "aggs": {"avg_attendees": {"avg": {"field": "attendees"}}},
                    "terms": {"field": "country"},
                }
            },
            "highlight": {"fields": {"title": {"fragment_size": 50}}, "order": "score"},
            "post_filter": {
                "bool": {"must": [{"terms": {"tags": ["prague", "czech"]}}]}
            },
            "query": {
                "bool": {
                    "filter": [
                        {
                            "bool": {
                                "should": [
                                    {"term": {"category": {"value": "meetup"}}},
                                    {"term": {"category": {"value": "conference"}}},
                                ]
                            }
                        }
                    ],
                    "must": [
                        {
                            "bool": {
                                "minimum_should_match": 2,
                                "must": [{"match": {"title": {"query": "python"}}}],
                                "must_not": [{"match": {"title": {"query": "ruby"}}}],
                            }
                        }
                    ],
                }
            },
            "script_fields": {
                "more_attendees": {"script": "doc['attendees'].value + 42"}
            },
            "size": 5,
            "sort": ["title", {"category": {"order": "desc"}}, "_score"],
            "suggest": {
                "my-title-suggestions-1": {
                    "term": {"field": "title", "size": 3},
                    "text": "devloping distibutd saerch " "engies",
                }
            },
        }

        d2 = deepcopy(d)

        s = Search.from_dict(d)

        # make sure we haven't modified anything in place
        self.assertEqual(d, d2)
        self.assertSearchEqual(d, s.to_dict())

    def test_from_dict_doesnt_need_query(self):
        s = Search.from_dict({"size": 5})

        assert {"size": 5} == s.to_dict()

    def test_source(self):
        assert {} == Search().source().to_dict()

        assert {
            "_source": {"includes": ["foo.bar.*"], "excludes": ["foo.one"]}
        } == Search().source(includes=["foo.bar.*"], excludes=["foo.one"]).to_dict()

        assert {"_source": False} == Search().source(False).to_dict()

        assert {"_source": ["f1", "f2"]} == Search().source(
            includes=["foo.bar.*"], excludes=["foo.one"]
        ).source(["f1", "f2"]).to_dict()

    def test_source_on_clone(self):
        self.assertEqual(
            {
                "_source": {"includes": ["foo.bar.*"], "excludes": ["foo.one"]},
                "query": {
                    "bool": {"filter": [{"term": {"title": {"value": "python"}}}],}
                },
            },
            Search()
            .source(includes=["foo.bar.*"])
            .source(excludes=["foo.one"])
            .filter("term", title="python")
            .to_dict(),
        )
        self.assertEqual(
            {
                "_source": False,
                "query": {
                    "bool": {"filter": [{"term": {"title": {"value": "python"}}}],}
                },
            },
            Search().source(False).filter("term", title="python").to_dict(),
        )

    def test_source_on_clear(self):
        assert (
            {}
            == Search()
            .source(includes=["foo.bar.*"])
            .source(includes=None, excludes=None)
            .to_dict()
        )

    def test_suggest_accepts_global_text(self):
        s = Search.from_dict(
            {
                "suggest": {
                    "text": "the amsterdma meetpu",
                    "my-suggest-1": {"term": {"field": "title"}},
                    "my-suggest-2": {"text": "other", "term": {"field": "body"}},
                }
            }
        )

        assert {
            "suggest": {
                "my-suggest-1": {
                    "term": {"field": "title"},
                    "text": "the amsterdma meetpu",
                },
                "my-suggest-2": {"term": {"field": "body"}, "text": "other"},
            }
        } == s.to_dict()

    def test_suggest(self):
        s = Search()
        s = s.suggest("my_suggestion", "pyhton", term={"field": "title"})

        assert {
            "suggest": {"my_suggestion": {"term": {"field": "title"}, "text": "pyhton"}}
        } == s.to_dict()

    def test_exclude(self):
        s = Search()
        s = s.exclude("match", title="python")

        self.assertEqual(
            {
                "query": {
                    "bool": {
                        "filter": [
                            {
                                "bool": {
                                    "must_not": [
                                        {"match": {"title": {"query": "python"}}}
                                    ]
                                }
                            }
                        ]
                    }
                }
            },
            s.to_dict(),
        )

    def test_update_from_dict(self):
        s = Search()
        s.update_from_dict({"indices_boost": [{"important-documents": 2}]})
        s.update_from_dict({"_source": ["id", "name"]})

        assert {
            "indices_boost": [{"important-documents": 2}],
            "_source": ["id", "name"],
        } == s.to_dict()



# your cluster address, if local probably 'localhost:9200'
ES_HOST = 'localhost:9300'

# where your table dumps are stored, and where serialized output will be written
DATA_DIR = '/Users/leonard/Alkemics/studies/initiatives/pandagg/data/imdb'
OUTPUT_FILE_NAME = 'serialized.json'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from unittest import TestCase

from pandagg.query import ScriptScore, PinnedQuery, Match


class SpecializedQueriesTestCase(TestCase):
    def test_script_score_clause(self):
        b1 = ScriptScore(
            query=Match(field="message", query="elasticsearch"),
            script={"source": "doc['likes'].value / 10 "},
        )

        b2 = ScriptScore(
            query={"match": {"message": "elasticsearch"}},
            script={"source": "doc['likes'].value / 10 "},
        )

        for b in (b1, b2):
            self.assertEqual(
                b.to_dict(),
                {
                    "script_score": {
                        "query": {"match": {"message": {"query": "elasticsearch"}}},
                        "script": {"source": "doc['likes'].value / 10 "},
                    }
                },
            )
            self.assertEqual(
                b.__repr__(),
                """<Query>
script_score, script={"source": "doc['likes'].value / 10 "}
└── query
    └── match, field=message, query="elasticsearch"
""",
            )

    def test_pinned_query_clause(self):
        b1 = PinnedQuery(
            ids=[1, 23], organic=Match(field="description", query="brown shoes")
        )

        b2 = PinnedQuery(ids=[1, 23], organic={"match": {"description": "brown shoes"}})

        for b in (b1, b2):
            self.assertEqual(
                b.to_dict(),
                {
                    "pinned": {
                        "ids": [1, 23],
                        "organic": {"match": {"description": {"query": "brown shoes"}}},
                    }
                },
            )
            self.assertEqual(
                b.__str__(),
                """<Query>
pinned, ids=[1, 23]
└── organic
    └── match, field=description, query="brown shoes"
""",
            )
